@@ -29,6 +29,10 @@ GType g_mount_mount_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_MOUNT_MOUNT_FLAGS (g_mount_mount_flags_get_type ())
 GType g_mount_unmount_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_MOUNT_UNMOUNT_FLAGS (g_mount_unmount_flags_get_type ())
+GType g_drive_start_flags_get_type (void) G_GNUC_CONST;
+#define G_TYPE_DRIVE_START_FLAGS (g_drive_start_flags_get_type ())
+GType g_drive_start_stop_type_get_type (void) G_GNUC_CONST;
+#define G_TYPE_DRIVE_START_STOP_TYPE (g_drive_start_stop_type_get_type ())
 GType g_file_copy_flags_get_type (void) G_GNUC_CONST;
 #define G_TYPE_FILE_COPY_FLAGS (g_file_copy_flags_get_type ())
 GType g_file_monitor_flags_get_type (void) G_GNUC_CONST;
