@@ -210,7 +210,7 @@ _g_local_file_output_stream_really_close (GLocalFileOutputStream *file,
       int errsv = errno;
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error writing to file: %s"),
 		   g_strerror (errsv));
       goto err_out;
@@ -867,7 +867,7 @@ handle_overwrite_open (const char    *filename,
       tmp_filename = g_build_filename (dirname, ".goutputstream-XXXXXX", NULL);
       g_free (dirname);
 
-      tmpfd = g_mkstemp (tmp_filename);
+      tmpfd = g_mkstemp_full (tmp_filename, (readable ? O_RDWR : O_WRONLY) | O_BINARY, mode);
       if (tmpfd == -1)
 	{
 	  g_free (tmp_filename);
