@@ -55,6 +55,7 @@ struct _GRealArray
   guint   elt_size;
   guint   zero_terminated : 1;
   guint   clear : 1;
+  volatile gint ref_count;
 };
 
 #define g_array_elt_len(array,i) ((array)->elt_size * (i))
@@ -91,6 +92,7 @@ GArray* g_array_sized_new (gboolean zero_terminated,
   array->zero_terminated = (zero_terminated ? 1 : 0);
   array->clear           = (clear ? 1 : 0);
   array->elt_size        = elt_size;
+  array->ref_count       = 1;
 
   if (array->zero_terminated || reserved_size != 0)
     {
@@ -101,23 +103,96 @@ GArray* g_array_sized_new (gboolean zero_terminated,
   return (GArray*) array;
 }
 
-gchar*
-g_array_free (GArray  *array,
-	      gboolean free_segment)
+/**
+ * g_array_ref:
+ * @array: A #GArray.
+ *
+ * Atomically increments the reference count of @array by one. This
+ * function is MT-safe and may be called from any thread.
+ *
+ * Returns: The passed in #GArray.
+ *
+ * Since: 2.22
+ **/
+GArray *
+g_array_ref (GArray *array)
 {
+  GRealArray *rarray = (GRealArray*) array;
+  g_return_val_if_fail (g_atomic_int_get (&rarray->ref_count) > 0, array);
+  g_atomic_int_inc (&rarray->ref_count);
+  return array;
+}
+
+/**
+ * g_array_unref:
+ * @array: A #GArray.
+ *
+ * Atomically decrements the reference count of @array by one. If the
+ * reference count drops to 0, all memory allocated by the array is
+ * released. This function is MT-safe and may be called from any
+ * thread.
+ *
+ * Since: 2.22
+ **/
+void
+g_array_unref (GArray *array)
+{
+  GRealArray *rarray = (GRealArray*) array;
+  g_return_if_fail (g_atomic_int_get (&rarray->ref_count) > 0);
+  if (g_atomic_int_dec_and_test (&rarray->ref_count))
+    g_array_free (array, TRUE);
+}
+
+/**
+ * g_array_get_element_size:
+ * @array: A #GArray.
+ *
+ * Gets the size of the elements in @array.
+ *
+ * Returns: Size of each element, in bytes.
+ *
+ * Since: 2.22
+ **/
+guint
+g_array_get_element_size (GArray *array)
+{
+  GRealArray *rarray = (GRealArray*) array;
+  return rarray->elt_size;
+}
+
+gchar*
+g_array_free (GArray   *farray,
+	      gboolean  free_segment)
+{
+  GRealArray *array = (GRealArray*) farray;
   gchar* segment;
+  gboolean preserve_wrapper;
 
   g_return_val_if_fail (array, NULL);
 
+  /* if others are holding a reference, preserve the wrapper but do free/return the data */
+  preserve_wrapper = FALSE;
+  if (g_atomic_int_get (&array->ref_count) > 1)
+    preserve_wrapper = TRUE;
+
   if (free_segment)
     {
       g_free (array->data);
       segment = NULL;
     }
   else
-    segment = array->data;
+    segment = (gchar*) array->data;
 
-  g_slice_free1 (sizeof (GRealArray), array);
+  if (preserve_wrapper)
+    {
+      array->data            = NULL;
+      array->len             = 0;
+      array->alloc           = 0;
+    }
+  else
+    {
+      g_slice_free1 (sizeof (GRealArray), array);
+    }
 
   return segment;
 }
@@ -164,7 +239,7 @@ g_array_prepend_vals (GArray        *farray,
 
 GArray*
 g_array_insert_vals (GArray        *farray,
-		     guint          index,
+		     guint          index_,
 		     gconstpointer  data,
 		     guint          len)
 {
@@ -172,11 +247,11 @@ g_array_insert_vals (GArray        *farray,
 
   g_array_maybe_expand (array, len);
 
-  g_memmove (g_array_elt_pos (array, len + index), 
-	     g_array_elt_pos (array, index), 
-	     g_array_elt_len (array, array->len - index));
+  g_memmove (g_array_elt_pos (array, len + index_), 
+	     g_array_elt_pos (array, index_), 
+	     g_array_elt_len (array, array->len - index_));
 
-  memcpy (g_array_elt_pos (array, index), data, g_array_elt_len (array, len));
+  memcpy (g_array_elt_pos (array, index_), data, g_array_elt_len (array, len));
 
   array->len += len;
 
@@ -197,10 +272,8 @@ g_array_set_size (GArray *farray,
       if (array->clear)
 	g_array_elt_zero (array, array->len, length - array->len);
     }
-#ifdef ENABLE_GC_FRIENDLY  
-  else if (length < array->len)
+  else if (G_UNLIKELY (g_mem_gc_friendly) && length < array->len)
     g_array_elt_zero (array, length, array->len - length);
-#endif /* ENABLE_GC_FRIENDLY */  
   
   array->len = length;
   
@@ -210,61 +283,59 @@ g_array_set_size (GArray *farray,
 }
 
 GArray*
-g_array_remove_index (GArray* farray,
-		      guint index)
+g_array_remove_index (GArray *farray,
+		      guint   index_)
 {
   GRealArray* array = (GRealArray*) farray;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index < array->len, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
 
-  if (index != array->len - 1)
-    g_memmove (g_array_elt_pos (array, index),
-	       g_array_elt_pos (array, index + 1),
-	       g_array_elt_len (array, array->len - index - 1));
+  if (index_ != array->len - 1)
+    g_memmove (g_array_elt_pos (array, index_),
+	       g_array_elt_pos (array, index_ + 1),
+	       g_array_elt_len (array, array->len - index_ - 1));
   
   array->len -= 1;
 
-#ifdef ENABLE_GC_FRIENDLY
-  g_array_elt_zero (array, array->len, 1);
-#else /* !ENABLE_GC_FRIENDLY */
-  g_array_zero_terminate (array);
-#endif /* ENABLE_GC_FRIENDLY */  
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    g_array_elt_zero (array, array->len, 1);
+  else
+    g_array_zero_terminate (array);
 
   return farray;
 }
 
 GArray*
-g_array_remove_index_fast (GArray* farray,
-			   guint   index)
+g_array_remove_index_fast (GArray *farray,
+			   guint   index_)
 {
   GRealArray* array = (GRealArray*) farray;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index < array->len, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
 
-  if (index != array->len - 1)
-    memcpy (g_array_elt_pos (array, index), 
+  if (index_ != array->len - 1)
+    memcpy (g_array_elt_pos (array, index_), 
 	    g_array_elt_pos (array, array->len - 1),
 	    g_array_elt_len (array, 1));
   
   array->len -= 1;
 
-#ifdef ENABLE_GC_FRIENDLY
-  g_array_elt_zero (array, array->len, 1);
-#else /* !ENABLE_GC_FRIENDLY */
-  g_array_zero_terminate (array);
-#endif /* ENABLE_GC_FRIENDLY */  
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    g_array_elt_zero (array, array->len, 1);
+  else
+    g_array_zero_terminate (array);
 
   return farray;
 }
 
 GArray*
-g_array_remove_range (GArray       *farray,
-                      guint         index_,
-                      guint         length)
+g_array_remove_range (GArray *farray,
+                      guint   index_,
+                      guint   length)
 {
   GRealArray *array = (GRealArray*) farray;
 
@@ -278,11 +349,10 @@ g_array_remove_range (GArray       *farray,
                (array->len - (index_ + length)) * array->elt_size);
 
   array->len -= length;
-#ifdef ENABLE_GC_FRIENDLY
-  g_array_elt_zero (array, array->len, length);
-#else /* !ENABLE_GC_FRIENDLY */
-  g_array_zero_terminate (array);
-#endif /* ENABLE_GC_FRIENDLY */
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    g_array_elt_zero (array, array->len, length);
+  else
+    g_array_zero_terminate (array);
 
   return farray;
 }
@@ -343,9 +413,8 @@ g_array_maybe_expand (GRealArray *array,
 
       array->data = g_realloc (array->data, want_alloc);
 
-#ifdef ENABLE_GC_FRIENDLY
-      memset (array->data + array->alloc, 0, want_alloc - array->alloc);
-#endif /* ENABLE_GC_FRIENDLY */
+      if (G_UNLIKELY (g_mem_gc_friendly))
+        memset (array->data + array->alloc, 0, want_alloc - array->alloc);
 
       array->alloc = want_alloc;
     }
@@ -358,9 +427,11 @@ typedef struct _GRealPtrArray  GRealPtrArray;
 
 struct _GRealPtrArray
 {
-  gpointer *pdata;
-  guint     len;
-  guint     alloc;
+  gpointer     *pdata;
+  guint         len;
+  guint         alloc;
+  volatile gint ref_count;
+  GDestroyNotify element_free_func;
 };
 
 static void g_ptr_array_maybe_expand (GRealPtrArray *array,
@@ -380,6 +451,8 @@ g_ptr_array_sized_new (guint reserved_size)
   array->pdata = NULL;
   array->len = 0;
   array->alloc = 0;
+  array->ref_count = 1;
+  array->element_free_func = NULL;
 
   if (reserved_size != 0)
     g_ptr_array_maybe_expand (array, reserved_size);
@@ -387,49 +460,146 @@ g_ptr_array_sized_new (guint reserved_size)
   return (GPtrArray*) array;  
 }
 
-gpointer*
-g_ptr_array_free (GPtrArray   *array,
-		  gboolean  free_segment)
+/**
+ * g_ptr_array_new_with_free_func:
+ * @element_free_func: A function to free elements with destroy @array or %NULL.
+ *
+ * Creates a new #GPtrArray with a reference count of 1 and use @element_free_func
+ * for freeing each element when the array is destroyed either via
+ * g_ptr_array_unref(), when g_ptr_array_free() is called with @free_segment
+ * set to %TRUE or when removing elements.
+ *
+ * Returns: A new #GPtrArray.
+ *
+ * Since: 2.22
+ **/
+GPtrArray *
+g_ptr_array_new_with_free_func (GDestroyNotify element_free_func)
 {
+  GPtrArray *array;
+
+  array = g_ptr_array_new ();
+  g_ptr_array_set_free_func (array, element_free_func);
+  return array;
+}
+
+/**
+ * g_ptr_array_set_free_func:
+ * @array: A #GPtrArray.
+ * @element_free_func: A function to free elements with destroy @array or %NULL.
+ *
+ * Sets a function for freeing each element when @array is destroyed
+ * either via g_ptr_array_unref(), when g_ptr_array_free() is called
+ * with @free_segment set to %TRUE or when removing elements.
+ *
+ * Since: 2.22
+ **/
+void
+g_ptr_array_set_free_func (GPtrArray        *array,
+                           GDestroyNotify    element_free_func)
+{
+  GRealPtrArray* rarray = (GRealPtrArray*) array;
+  rarray->element_free_func = element_free_func;
+}
+
+/**
+ * g_ptr_array_ref:
+ * @array: A #GArray.
+ *
+ * Atomically increments the reference count of @array by one. This
+ * function is MT-safe and may be called from any thread.
+ *
+ * Returns: The passed in #GPtrArray.
+ *
+ * Since: 2.22
+ **/
+GPtrArray *
+g_ptr_array_ref (GPtrArray *array)
+{
+  GRealPtrArray *rarray = (GRealPtrArray*) array;
+  g_return_val_if_fail (g_atomic_int_get (&rarray->ref_count) > 0, array);
+  g_atomic_int_inc (&rarray->ref_count);
+  return array;
+}
+
+/**
+ * g_ptr_array_unref:
+ * @array: A #GPtrArray.
+ *
+ * Atomically decrements the reference count of @array by one. If the
+ * reference count drops to 0, the effect is the same as calling
+ * g_ptr_array_free() with @free_segment set to %TRUE. This function
+ * is MT-safe and may be called from any thread.
+ *
+ * Since: 2.22
+ **/
+void
+g_ptr_array_unref (GPtrArray *array)
+{
+  GRealPtrArray *rarray = (GRealPtrArray*) array;
+  g_return_if_fail (g_atomic_int_get (&rarray->ref_count) > 0);
+  if (g_atomic_int_dec_and_test (&rarray->ref_count))
+    g_ptr_array_free (array, TRUE);
+}
+
+gpointer*
+g_ptr_array_free (GPtrArray *farray,
+		  gboolean   free_segment)
+{
+  GRealPtrArray *array = (GRealPtrArray*) farray;
   gpointer* segment;
+  gboolean preserve_wrapper;
 
   g_return_val_if_fail (array, NULL);
 
+  /* if others are holding a reference, preserve the wrapper but do free/return the data */
+  preserve_wrapper = FALSE;
+  if (g_atomic_int_get (&array->ref_count) > 1)
+    preserve_wrapper = TRUE;
+
   if (free_segment)
     {
+      if (array->element_free_func != NULL)
+        g_ptr_array_foreach (farray, (GFunc) array->element_free_func, NULL);
       g_free (array->pdata);
       segment = NULL;
     }
   else
     segment = array->pdata;
 
-  g_slice_free1 (sizeof (GRealPtrArray), array);
+  if (preserve_wrapper)
+    {
+      array->pdata = NULL;
+      array->len = 0;
+      array->alloc = 0;
+    }
+  else
+    {
+      g_slice_free1 (sizeof (GRealPtrArray), array);
+    }
 
   return segment;
 }
 
 static void
 g_ptr_array_maybe_expand (GRealPtrArray *array,
-			  gint        len)
+			  gint           len)
 {
   if ((array->len + len) > array->alloc)
     {
-#ifdef ENABLE_GC_FRIENDLY
       guint old_alloc = array->alloc;
-#endif /* ENABLE_GC_FRIENDLY */
       array->alloc = g_nearest_pow (array->len + len);
       array->alloc = MAX (array->alloc, MIN_ARRAY_SIZE);
       array->pdata = g_realloc (array->pdata, sizeof (gpointer) * array->alloc);
-#ifdef ENABLE_GC_FRIENDLY
-      for ( ; old_alloc < array->alloc; old_alloc++)
-	array->pdata [old_alloc] = NULL;
-#endif /* ENABLE_GC_FRIENDLY */
+      if (G_UNLIKELY (g_mem_gc_friendly))
+        for ( ; old_alloc < array->alloc; old_alloc++)
+          array->pdata [old_alloc] = NULL;
     }
 }
 
 void
-g_ptr_array_set_size  (GPtrArray   *farray,
-		       gint	     length)
+g_ptr_array_set_size  (GPtrArray *farray,
+		       gint	  length)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
 
@@ -448,98 +618,105 @@ g_ptr_array_set_size  (GPtrArray   *farray,
       for (i = array->len; i < length; i++)
 	array->pdata[i] = NULL;
     }
-#ifdef ENABLE_GC_FRIENDLY  
   else if (length < array->len)
-    {
-      int i;
-      for (i = length; i < array->len; i++)
-	array->pdata[i] = NULL;
-    }
-#endif /* ENABLE_GC_FRIENDLY */  
+    g_ptr_array_remove_range (farray, length, array->len - length);
 
   array->len = length;
 }
 
 gpointer
-g_ptr_array_remove_index (GPtrArray* farray,
-			  guint      index)
+g_ptr_array_remove_index (GPtrArray *farray,
+			  guint      index_)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   gpointer result;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index < array->len, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
 
-  result = array->pdata[index];
+  result = array->pdata[index_];
   
-  if (index != array->len - 1)
-    g_memmove (array->pdata + index, array->pdata + index + 1, 
-	       sizeof (gpointer) * (array->len - index - 1));
+  if (array->element_free_func != NULL)
+    array->element_free_func (array->pdata[index_]);
+
+  if (index_ != array->len - 1)
+    g_memmove (array->pdata + index_, array->pdata + index_ + 1, 
+               sizeof (gpointer) * (array->len - index_ - 1));
   
   array->len -= 1;
 
-#ifdef ENABLE_GC_FRIENDLY  
-  array->pdata[array->len] = NULL;
-#endif /* ENABLE_GC_FRIENDLY */  
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    array->pdata[array->len] = NULL;
 
   return result;
 }
 
 gpointer
-g_ptr_array_remove_index_fast (GPtrArray* farray,
-			       guint      index)
+g_ptr_array_remove_index_fast (GPtrArray *farray,
+			       guint      index_)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   gpointer result;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index < array->len, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
 
-  result = array->pdata[index];
+  result = array->pdata[index_];
   
-  if (index != array->len - 1)
-    array->pdata[index] = array->pdata[array->len - 1];
+  if (index_ != array->len - 1)
+    {
+      if (array->element_free_func != NULL)
+        array->element_free_func (array->pdata[index_]);
+      array->pdata[index_] = array->pdata[array->len - 1];
+    }
 
   array->len -= 1;
 
-#ifdef ENABLE_GC_FRIENDLY  
-  array->pdata[array->len] = NULL;
-#endif /* ENABLE_GC_FRIENDLY */  
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    array->pdata[array->len] = NULL;
 
   return result;
 }
 
 void
-g_ptr_array_remove_range (GPtrArray* farray,
+g_ptr_array_remove_range (GPtrArray *farray,
                           guint      index_,
                           guint      length)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
+  guint n;
 
   g_return_if_fail (array);
   g_return_if_fail (index_ < array->len);
   g_return_if_fail (index_ + length <= array->len);
 
+  if (array->element_free_func != NULL)
+    {
+      for (n = index_; n < index_ + length; n++)
+        array->element_free_func (array->pdata[n]);
+    }
+
   if (index_ + length != array->len)
-    g_memmove (&array->pdata[index_],
-               &array->pdata[index_ + length], 
-               (array->len - (index_ + length)) * sizeof (gpointer));
+    {
+      g_memmove (&array->pdata[index_],
+                 &array->pdata[index_ + length], 
+                 (array->len - (index_ + length)) * sizeof (gpointer));
+    }
 
   array->len -= length;
-#ifdef ENABLE_GC_FRIENDLY
-  {
-    guint i;
-    for (i = 0; i < length; i++)
-      array->pdata[array->len + i] = NULL;
-  }
-#endif /* ENABLE_GC_FRIENDLY */
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    {
+      guint i;
+      for (i = 0; i < length; i++)
+        array->pdata[array->len + i] = NULL;
+    }
 }
 
 gboolean
-g_ptr_array_remove (GPtrArray* farray,
-		    gpointer data)
+g_ptr_array_remove (GPtrArray *farray,
+		    gpointer   data)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   guint i;
@@ -559,8 +736,8 @@ g_ptr_array_remove (GPtrArray* farray,
 }
 
 gboolean
-g_ptr_array_remove_fast (GPtrArray* farray,
-			 gpointer data)
+g_ptr_array_remove_fast (GPtrArray *farray,
+			 gpointer   data)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   guint i;
@@ -580,8 +757,8 @@ g_ptr_array_remove_fast (GPtrArray* farray,
 }
 
 void
-g_ptr_array_add (GPtrArray* farray,
-		 gpointer data)
+g_ptr_array_add (GPtrArray *farray,
+		 gpointer   data)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
 
@@ -644,7 +821,7 @@ g_ptr_array_foreach (GPtrArray *array,
 /* Byte arrays 
  */
 
-GByteArray* g_byte_array_new      (void)
+GByteArray* g_byte_array_new (void)
 {
   return (GByteArray*) g_array_sized_new (FALSE, FALSE, 1, 0);
 }
@@ -660,18 +837,52 @@ guint8*	    g_byte_array_free     (GByteArray *array,
   return (guint8*) g_array_free ((GArray*) array, free_segment);
 }
 
-GByteArray* g_byte_array_append   (GByteArray *array,
+/**
+ * g_byte_array_ref:
+ * @array: A #GByteArray.
+ *
+ * Atomically increments the reference count of @array by one. This
+ * function is MT-safe and may be called from any thread.
+ *
+ * Returns: The passed in #GByteArray.
+ *
+ * Since: 2.22
+ **/
+GByteArray *
+g_byte_array_ref (GByteArray *array)
+{
+  return (GByteArray *) g_array_ref ((GArray *) array);
+}
+
+/**
+ * g_byte_array_unref:
+ * @array: A #GByteArray.
+ *
+ * Atomically decrements the reference count of @array by one. If the
+ * reference count drops to 0, all memory allocated by the array is
+ * released. This function is MT-safe and may be called from any
+ * thread.
+ *
+ * Since: 2.22
+ **/
+void
+g_byte_array_unref (GByteArray *array)
+{
+  g_array_unref ((GArray *) array);
+}
+
+GByteArray* g_byte_array_append   (GByteArray   *array,
 				   const guint8 *data,
-				   guint       len)
+				   guint         len)
 {
   g_array_append_vals ((GArray*) array, (guint8*)data, len);
 
   return array;
 }
 
-GByteArray* g_byte_array_prepend  (GByteArray *array,
+GByteArray* g_byte_array_prepend  (GByteArray   *array,
 				   const guint8 *data,
-				   guint       len)
+				   guint         len)
 {
   g_array_prepend_vals ((GArray*) array, (guint8*)data, len);
 
@@ -687,25 +898,25 @@ GByteArray* g_byte_array_set_size (GByteArray *array,
 }
 
 GByteArray* g_byte_array_remove_index (GByteArray *array,
-				       guint index)
+				       guint       index_)
 {
-  g_array_remove_index((GArray*) array, index);
+  g_array_remove_index ((GArray*) array, index_);
 
   return array;
 }
 
 GByteArray* g_byte_array_remove_index_fast (GByteArray *array,
-					    guint index)
+					    guint       index_)
 {
-  g_array_remove_index_fast((GArray*) array, index);
+  g_array_remove_index_fast ((GArray*) array, index_);
 
   return array;
 }
 
 GByteArray*
 g_byte_array_remove_range (GByteArray *array,
-                           guint index_,
-                           guint length)
+                           guint       index_,
+                           guint       length)
 {
   g_return_val_if_fail (array, NULL);
   g_return_val_if_fail (index_ < array->len, NULL);
